from django.db.models.fields.files import FieldFile


def get_ext(filename):
	filename = filename.split('.')
	filetype = filename[len(filename) - 1].lower()
	if filetype == 'jpeg':
		filetype = 'jpg'
	return filetype

def get_unique(instance):
	if instance.id:
		instance_id = instance.id
	else:
		Model = instance.__class__
		if Model.objects.all():
			last = Model.objects.all().order_by('-id')[0]
			instance_id = last.id + 1
		else:
			instance_id = 1

	if hasattr(instance, 'slug'):
		if instance._meta.get_field('slug').unique:
			return instance.slug
		else:
			return '%s-%s' % (instance_id, instance.slug)
	else:
		return '%s' % instance_id


def get_field(instance, filename):
	file_fields = []
	for field_name, _ in instance.__dict__.items():
		field = getattr(instance, field_name)
		if issubclass(field.__class__, FieldFile):
			file_fields.append(field_name)
		if field == filename:
			return field_name


def upload_to(instance, filename):
	puth = ''

	if hasattr(instance, 'dependent_from'):
		dependent_from = instance.dependent_from()
		puth += dependent_from._meta.app_label + '/'
		puth += dependent_from._meta.model_name + '/'
		puth += get_unique(dependent_from) + '/'

		# instance._meta.app_label
		puth += instance._meta.model_name + '/'

		puth += get_unique(instance) + '/'
		puth += get_field(instance, filename) + '.'
	else:
		puth += instance._meta.app_label + '/'
		puth += instance._meta.model_name + '/'
		puth += get_unique(instance) + '/'
		puth += get_field(instance, filename) + '.'

	puth += get_ext(filename)
	return puth



import re
import hashlib
import pynliner
import os

from email.mime.image import MIMEImage

from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.conf import settings


def mail(subject, context, template, from_email, to_email, connection=None, reply_to=None, attach_files=[], cc=None, bcc=None):

	html_template = render_to_string(template + '.html', context)
	txt_template = render_to_string(template + '.txt', context)

	# email debug mode
	try:
		email_mode = settings.HELPFUL_EMAIL_MODE
		email_debug_info = settings.HELPFUL_EMAIL_DEBUG_INFO
		email_debug_to_email = settings.HELPFUL_EMAIL_DEBUG_TO_EMAIL
	except:
		return

	if email_mode == 'PROD':
		pass
	elif email_mode == 'DEBUG':
		if email_debug_info:
			pre_info = 'to_email: ' + ', '.join(to_email) + '\n' \
					   + 'cc: ' + ', '.join(cc) + '\n' \
					   + 'bcc: ' + ', '.join(bcc)
			html_template = '<pre>' + pre_info + '\n\n</pre>' + html_template
			txt_template = pre_info + '\n\n' + txt_template
		to_email = email_debug_to_email
		cc = []
		bcc = []
	elif email_mode == 'BLANK':
		return
	else:
		return

	plinper = pynliner.Pynliner()
	plinper.relative_url = 'file://localhost/'

	html_email = plinper.from_string(html_template).run()

	email = EmailMultiAlternatives(subject, txt_template, from_email, to_email, reply_to=reply_to, cc=cc, bcc=bcc)
	email.mixed_subtype = 'related'

	# Find all images in html
	img_regex = '<img(.*?)src=("|\')(.*?)("|\')(.*?)>'
	compiled_img_regex = re.compile(img_regex)
	images = compiled_img_regex.findall(html_email)

	# Replace src to cid
	for image in images:
		img_path = image[2]
		img_file = open(img_path , 'rb')
		img = MIMEImage(img_file.read())
		img_file.close()

		img_cid = hashlib.md5(open(img_path, 'rb').read()).hexdigest()
		img.add_header('Content-ID', '<%s>' % img_cid)
		img.add_header('Content-Disposition', 'inline')

		email.attach(img)
		html_email = re.sub(img_path, 'cid:%s' % img_cid , html_email)

	for attach_file in attach_files:
		attachment = open(attach_file, 'rb')
		file_name = os.path.basename(attach_file)
		email.attach(file_name, attachment.read()) # , 'text/csv'

	email.attach_alternative(html_email, "text/html")

	if connection:
		connection.open()

	email.send()



from django.http import HttpResponseRedirect
from django.conf import settings

class LoginRequiredMiddleware(object):
	def __init__(self, get_response):
		self.get_response = get_response

	def __call__(self, request):
		response = self.get_response(request)

		if not request.user.is_authenticated:
			path = request.path_info.lstrip('/')

			if path not in ['accounts/login/', 'admin/', 'admin/login/?next=/admin/', 'admin/login/']:
				return HttpResponseRedirect(settings.LOGIN_URL)

		return response



from django import template

register = template.Library()

@register.filter
def get_item(obj, key):
	if hasattr(obj, key):
		return getattr(obj, key)
	else:
		return obj.__dict__.get(key)

@register.filter
def get_property_value(value, key):
	return value.get_property_value(key)



import os

from django.conf import settings

from django import template
register = template.Library()


@register.simple_tag(takes_context=True)
def abs_puth(context, place, file_name):
	if place == 'static':
		return os.path.abspath(settings.STATIC_ROOT + '/' + file_name)
	elif place == 'media':
		return os.path.abspath(settings.MEDIA_ROOT + '/' + file_name)
	elif place == 'base':
		return os.path.abspath(settings.BASE_DIR + '/' + file_name)
	elif place == 'parent':
		return os.path.abspath(settings.BASE_DIR + '/../' + file_name)



from django import template
register = template.Library()


def get_br(obj):
	br = [obj]
	if hasattr(obj, 'parent') and obj.parent:
		br += get_br(obj.parent)
	elif hasattr(obj, 'category') and obj.category:
		br += get_br(obj.category)
	return br

@register.simple_tag(takes_context=True)
def breadcrumbs(context, obj):
	context = {}
	context['breadcrumbs'] = reversed(get_br(obj))

	tpl = template.loader.get_template('helpful/breadcrumbs.html')
	return tpl.render(context)



from django.utils.translation import gettext_lazy as _
from django import template


register = template.Library()


def get_display(object, field):
	value = getattr(object, field.name)
	display = dict(field.flatchoices).get(value, value)
	if display is True:
		display = _('Yes')
	elif display is False:
		display = _('No')
	return display


@register.filter(name='object_dict')
def object_dict(object):
	fields = []
	for field in object._meta.fields:
		data = {}
		data['verbose_name'] = field.verbose_name
		data['display'] = get_display(object, field)
		data['field'] = field
		fields.append(data)
	return fields



import os
from django.conf import settings

from django.db.models.fields.files import FieldFile
from django.db.models.signals import pre_delete, pre_save
from django.dispatch import receiver


if hasattr(settings, 'MEDIA_TRASH'):
	def move2trash(field):
		old_path = field.path
		new_path = settings.MEDIA_TRASH + field.name

		trash_dir = os.path.dirname(new_path)

		try:
			os.makedirs(trash_dir, mode=0o777)
		except:
			pass

		os.rename(old_path, new_path)

	@receiver(pre_delete)
	def on_delete(sender, instance, *args, **kwargs):
		try:
			previos = sender.objects.get(id=instance.id)

			for field_name, _ in instance.__dict__.iteritems():
				field = getattr(instance, field_name)
				if issubclass(field.__class__, FieldFile):
					previos_file = getattr(previos, field_name)
					move2trash(previos_file)
		except:
			pass


	@receiver(pre_save)
	def on_change(sender, instance, *args, **kwargs):
		try:
			previos = sender.objects.get(id=instance.id)

			for field_name, _ in instance.__dict__.iteritems():
				field = getattr(instance, field_name)
				if issubclass(field.__class__, FieldFile):
					previos_file = getattr(previos, field_name)
					if previos_file != field:
						move2trash(previos_file)
		except:
			pass



from setuptools import setup, find_packages

# python3 -m pip install --upgrade twine
# python3 setup.py sdist bdist_wheel
# python3 -m twine upload --repository pypi dist/*
# rm -r build/
# rm -r dist/
# rm -r django_helpful.egg-info


setup(
    name="django-helpful",
    version="1.6.4",
    description="Django helpful things",
    long_description=open("README.rst").read(),
    author="Stanislav Baltrunas",
    author_email="stanislav@baltrunas.ru",
    license="BSD",
    url="https://github.com/Baltrunas/django-helpful",
    # package_dir={"": "helpful"},
    packages=["helpful", "helpful.templatetags"],
    package_data={"templates": ["*"], "static": ["*"], "locale": ["*"], "docs": ["*"],},
    include_package_data=True,
    install_requires=[],
    python_requires=">=2.6, <4",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Framework :: Django",
    ],
    zip_safe=False,
)



from django import template
from django.utils.safestring import mark_safe
from django.conf import settings


register = template.Library()


@register.simple_tag(takes_context=True)
def render(context, content):
    try:
        tpl = template.Template(content)
        content = template.Context(context)
        return tpl.render(content)
    except Exception as e:
        if settings.DEBUG:
            raise
        else:
            return mark_safe("<div class='error'>Render Error: <code>%s</code></div>" % e.message)
